#!/usr/bin/env python

"""
This demo attempts to override the C++ MainLoop and implement it
in Python.
"""
import sys

import wx
import wx.lib.newevent
from checkpanel import CheckPanel
from metricspanel import MetricsPanel
from pwrbox import PowerBox
import conf


class TabSwitchedPorts(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)


class TabPowerMetrics(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)


class MyFrame(wx.Frame):

    def __init__(self, parent, fid, title):
        wx.Frame.__init__(self, parent, -1, "Telescope Power Control",
                          size=(525, 400))

        self.Bind(wx.EVT_IDLE, self.OnIdle)

        self.statusbar = self.CreateStatusBar(2)
        self.statusbar.SetStatusWidths([200, -1])
        self.statusbar.SetStatusText("", 0)
        self.statusbar.SetStatusText("", 1)
        self.tab_panel = wx.Panel(self)
        self.notebook = wx.Notebook(self.tab_panel, size=(525, -1))

        self.pwrbox = PowerBox(self)

        self.switched = TabSwitchedPorts(self.notebook)
        self.metrics = TabPowerMetrics(self.notebook)

        self.left_panel = CheckPanel(self.switched, self.pwrbox,
                                     conf.pwr_items, "name")
        self.right_panel = CheckPanel(self.switched, self.pwrbox,
                                      conf.pwr_items, "auto")

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.left_panel, proportion=0, flag=wx.EXPAND, border=2)
        sizer.Add(self.right_panel, proportion=0, flag=wx.EXPAND, border=2)
        self.switched.SetSizer(sizer)

        self.metrics_panel = MetricsPanel(self.metrics, self.pwrbox,
                                          conf.pwr_items)

        sizer = wx.BoxSizer()
        sizer.Add(self.metrics_panel, proportion=0, flag=wx.EXPAND, border=2)
        self.metrics.SetSizer(sizer)

        self.notebook.AddPage(self.switched, "Switches")
        self.notebook.AddPage(self.metrics, "Metrics")

        sizer = wx.BoxSizer()
        sizer.Add(self.notebook, proportion=0, flag=wx.EXPAND, border=2)
        self.tab_panel.SetSizer(sizer)
        self.SetMinSize(wx.Size(525, 400))
        self.CenterOnScreen()
        self.Show()
        self.statusbar.SetStatusText(self.pwrbox.port, 1)
        for chnl in range(0, 8):
            if self.pwrbox.get_chnl_state(chnl):
                conf.pwr_items[chnl]["state"] = "On"
            else:
                conf.pwr_items[chnl]["state"] = "Off"
        for chnl in range(0, 8):
            if self.pwrbox.get_auto_pwron(chnl):
                conf.pwr_items[chnl]["auto"] = "Auto"
            else:
                conf.pwr_items[chnl]["auto"] = "Manual"

        self.left_panel.update_gui(conf.pwr_items)
        self.right_panel.update_gui(conf.pwr_items)
        # self.heater_panel.update_gui(conf.htr_items)
        # self.metrics_panel.update_gui()
        self.count = 0

    def OnCloseWindow(self, event):
        self.Destroy()

    def OnIdle(self, event):
        pass

    def OnSize(self, event):
        size = event.GetSize()
        event.Skip()

    def OnMove(self, event):
        pos = event.GetPosition()


class MyEventLoop(wx.GUIEventLoop):
    def __init__(self):
        wx.GUIEventLoop.__init__(self)
        self.exitCode = 0
        self.shouldExit = False

    def DoMyStuff(self):
        # Do whatever you want to have done for each iteration of the event
        # loop. In this example we'll just sleep a bit to simulate something
        # real happening.

        evt = wx.IdleEvent()
        wx.PostEvent(app.frame.GetEventHandler(), evt)
        pass

    def Run(self):
        # Set this loop as the active one. It will automatically reset to the
        # original evtloop when the context manager exits.
        with wx.EventLoopActivator(self):
            while True:

                self.DoMyStuff()

                # Generate and process idles events for as long as there
                # isn't anything else to do
                while not self.shouldExit and not self.Pending() and self.ProcessIdle():
                    pass

                if self.shouldExit:
                    break

                # Dispatch all the pending events
                self.ProcessEvents()

                # Currently on wxOSX Pending always returns true, so the
                # ProcessIdle above is not ever called. Call it here instead.
                if 'wxOSX' in wx.PlatformInfo:
                    self.ProcessIdle()

            # Process remaining queued messages, if any
            while True:
                checkAgain = False
                if wx.GetApp() and wx.GetApp().HasPendingEvents():
                    wx.GetApp().ProcessPendingEvents()
                    checkAgain = True
                if 'wxOSX' not in wx.PlatformInfo and self.Pending():
                    self.Dispatch()
                    checkAgain = True
                if not checkAgain:
                    break

        return self.exitCode

    def Exit(self, rc=0):
        self.exitCode = rc
        self.shouldExit = True
        self.OnExit()
        self.WakeUp()

    def ProcessEvents(self):
        if wx.GetApp():
            wx.GetApp().ProcessPendingEvents()

        if self.shouldExit:
            return False

        return self.Dispatch()


class MyApp(wx.App):

    def MainLoop(self):
        self.SetExitOnFrameDelete(True)
        self.mainLoop = MyEventLoop()
        self.mainLoop.Run()

    def ExitMainLoop(self):
        self.mainLoop.Exit()

    def OnInit(self):
        self.frame = MyFrame(None, sys.stdout, "Power Control")
        self.frame.Show(True)
        self.SetTopWindow(self.frame)

        return True


app = MyApp(False)
app.MainLoop()
